function xF = find_periodic_1d(fhandle, period, interval, init_num, tolerance, diverge, max_step, rnd, varargin)

%
% Finding periodic points of the specified period, probing the defined
% number of initial conditions, which are either set randomly or
% distributed regularly in the given interval
% 
% fhandle   - function handle 
% period    - period of points to search
% interval  - interval for searching perioodic points 
% init_num  - the number of initial conditions to try
% tolerance - checking point accuracy
% max_step  - maximal number of Newton steps exceeding this
%             value means that the Newton method do not
%             converge 
% rnd       - set initial conditions randomly or not
%             if true (or 1), then initial conditions are
%             chosen randomly (in the total amount of
%             init_num) if false (or 0), then the search
%             interval is divided into init_num equal
%             subintervals, and a middle point of each
%             subinterval is taken for the initial condition
% varargin  - cell array of the map parameters
%
% return values:
% xF - the vector of periodic points found sorted in
%      ascending order 
%

%%% TODO
%%% Needs checking for divergence


%diverge = 100000;



dim = 1; % this routine is only for 1D maps
xF = [];

% checking if there exists an explicit Jacobi matrix function
fh = functions(fhandle);
Jname = ['@' fh.function '_J'];
Jhandle = str2func(Jname);
Jh = functions(Jhandle);
if strcmp(Jh.file, '')   % there is no file with Jacobi matrix function
    cprintf(get_colour('warn'), 'WARNING: The function derivative is not defined\n');
    cprintf(get_colour('warn'), '         and will be calculated approximately.\n\n');
    has_diff = 0;
else
    has_diff = 1;
end

if (init_num == 0)
    cprintf(get_colour('vital'), 'Using the predefined initial condition(s).\n\n');
    total = length(interval);
else
    sz = size(interval);
    if (sz(2) ~= 2)
        cprintf('err', 'ERROR: interval (arg #3) must be a vector with two columns.\n');
        return;
    end
    
    if (~rnd)
        delta = (interval(2) - interval(1))/init_num;
    end
    
    total = init_num;
end
    
I = eye(dim, dim);


% applying Newton method for each initial condition
x_all = [];
for j = 1:total

    cprintf(get_colour('aux'), 'Run number %d\n', j);
    
    if (init_num == 0)
        x0 = interval(j);
    else
        if (rnd) % choose randomly
            x0 = interval(1) + (interval(2) - interval(1))*rand;
        else % take x0 as a middle point of a subinterval
            x0 = interval(1) + (j - 0.5)*delta;
        end
    end;

    
    x_init = x0;

    k = 1;
    while (k <= max_step)
        % iterate the function period times to get the final Jacobian
        if (has_diff)
            J = Jhandle(x0, varargin{:});
        else
            J = jacob(fhandle, x0, 0.0001, varargin{:});
        end
        x_iter = x0;
        for i = 2:period
            x_iter = fhandle(x_iter, varargin{:});
            if (has_diff)
                J = Jhandle(x_iter, varargin{:})*J;
            else
                J = jacob(fhandle, x_iter, 0.0001, varargin{:})*J;
            end
        end
        % x and f(x) are row vectors, thus, we need transpose operations here
        dx = (J - I)\(fhandle(x_iter, varargin{:}) - x0)';
    
        % new value of solution approximation
        x1 = x0' - dx;
        % period iterate of function for new approximation
        x_iter = x1;
        for i = 1:period
            x_iter = fhandle(x_iter, varargin{:});
        end
    
        % stop if error is small enough
        if norm(x_iter - x1) < tolerance
            x_all = [x_all; x1'];
            cprintf(get_colour('vital'), 'Total number of Newton steps is %d.\n\n', k);
            break;
        end
    
        x0 = x1';
        k = k + 1;
    end
    
    
    if (k > max_step)
        cprintf(get_colour('warn'), 'WARNING: The Newton method failed to converge.\n');
        cprintf(get_colour('warn'), '         The solution has not been found.\n');
        cprintf(get_colour('warn'), '         Initial condition was %.8f.\n\n', x_init);
    end

end


cprintf(get_colour('final'), 'The total number of initial conditions checked is %d.\n', total);
if (isempty(x_all))
    cprintf(get_colour('final'), 'No points were found.\n');
else
    % eliminating repeated values
    % sorting found solutions in increasing order
    x_all = sort(x_all);
    m = length(x_all);
    xF = [x_all(1, :)];
    for i = 2:m
        % if the adjusent sollutions are not close
        % they should be different periodic points
        if (norm(x_all(i, :) - x_all(i-1, :)) > tolerance*10)
            xF = [xF; x_all(i, :)];
        end
    end

    cprintf(get_colour('final'), 'The total number of distinct points found is %d.\n', ...
            length(xF(:, 1)));
end



