function [p, arc_length] = sc_stable_cyc(fhandle, p0, period, arc_max, max_iter, d, ...
    a_min, a_max, Da_min, Da_max, D_min, eB, diverge, varargin)

%
% Find a 1D stable manifold of a saddle fixed point
% by using a search circle algorithm
% (see England & Krauskopf & Osinga, SIAM JADS 3(2): 161-190, 2004)
% periodic points
%
% fhandle        - function (dynamical system) handle
% p0             - saddle fixed point
% period         - period of the target saddle point
% arc_max        - desired length of the manifold arc
% max_iter       - maximal iterates number to try
% d              - initial step size
%                  (if d > 0 manifold is grown forward, if d
%                  < 0 manifold is grown backward) 
% a_min, a_max   - angle bounds (a_min < ak < a_max)
% Da_min, Da_max - bounds for ak*Dk (da_min < ak*Dk < Da_max)
% D_min          - minimal step size
% eB             - bisection error
% varargin       - cell array of the function parameters
%
% return values:
% p          - array of the computed manifold points
% arc_length - length of the computed manifold arc 
%

% diverge = 100000;

load('data/default.mat', 'da'); % angle increment for finding correct p_start, p_end

% half of the minimal step size
half_D_min = D_min/2;

p = [];
arc_length = 0;

p(1, :) = p0;

dx = 0.0001;
J = calc_jacob_cyc(fhandle, p0, period, dx, varargin{:})

[v, l] = eig(J);
l = diag(l);
cprintf(get_colour('aux'), '\n');
cprintf(get_colour('aux'), 'Eigenvalues are %.8f and %.8f\n\n', l(1), l(2));
idx = find((abs(l) < 1) & (abs(l) ~= 0));
if (length(idx) ~= 1)
    cprintf('err', 'ERROR: The routine finds only one-dimensional manifolds.\n');
    return;
end;


p(2, :) = (p0' + d*v(:, idx))';
arc_length = abs(d);


if (l(idx) > 0)
    %min_iter = 1;
    min_iter = period;
else
    %min_iter = 2;
    min_iter = 2*period;
end;



k = 2;
Dk = abs(d)*(1/abs(l(idx)) - 1);
iters = min_iter;

% Grow initial section: p_k \approx f(p_{k+1})
% continues until a_k < a_max, D_k*a_k < Da_max
while (arc_length < arc_max)
    
    % find Dk given its current estimate 
    Dk = find_delta(fhandle, p(k-1, :), p(k, :), Dk, iters, eB, diverge, varargin{:});
    
    % with found Dk do search circle
    [p_next, f_next, stat] = search_circle(fhandle, iters, p(k-1, :), p(k, :), ...
        p(k-1, :), p(k, :), Dk, a_max, eB, diverge, varargin{:});
    
    if (stat == -1)
        cprintf(get_colour('warn'), 'WARNING: alpha search range not found.\n');
        break;
    elseif (stat == 1)
        cprintf(get_colour('warn'), 'WARNING: alpha search range was enlarged.\n');
    elseif (stat == 2)
        cprintf(get_colour('warn'), 'WARNING: bisection accuracy was not reached.\n');
    end

    % the angle between the vectors [p_{k-1}, p_k] and [p_k, p_next]
    ak = vector_angle(p(k, :) - p(k-1, :), p_next - p(k, :));
    
    if ((ak < a_max) && (Dk*ak < Da_max))
        % the point is acceptable
        p(k+1, :) = p_next;
        arc_length = arc_length + Dk;
        
        k = k + 1;
    else
        % stop initial section
        cprintf(get_colour('vital'), 'Finished initial section, arc length is %.8f.\n', ...
                arc_length);
        break;
    end % if ((ak < a_max) && (Dk*ak < Da_max))

end % while (arc_length < arc_max), initial section




% Grow main section: f(p_{k+1}) \in [p_{i-1}, p_i]
% for some i
i = k;

while (arc_length < arc_max)
    
    if (mod(k, 1000) == 0)
        cprintf(get_colour('vital'), 'Computed upto k = %d. Arc length is %.8f.\n', ...
                k, arc_length);
    end;
    
    if(k == 2866)
        cprintf(get_colour('vital'), 'Computed upto k = %d. Critical value. Debugging...');
    end;
    

    % search direction, 1 if forward and -1 if backward
    dirctn = 0;

    % current number of iterations
    iters_cur = iters;
    % saving i for potential further computation with halved step size
    i_old = i;
    
    % checking if step size is too small
    if (Dk < D_min)
        Dk = D_min;
    end
    

    while (true)
        % try search circle with previous Dk
        [p_next, f_next, stat] = search_circle(fhandle, iters, p(k-1, :), p(k, :), ...
            p(i-1, :), p(i, :), Dk, a_max, eB, diverge, varargin{:});
        
        if (stat == -1)
            cprintf(get_colour('warn'), 'WARNING: alpha search range not found.\n');
        elseif (stat == 1)
            cprintf(get_colour('warn'), 'WARNING: alpha search range was enlarged.\n');
        elseif (stat == 2)
            cprintf(get_colour('warn'), 'WARNING: bisection accuracy was not reached.\n');
        end
    
        % check_segment
        right_i = check_segment(f_next, p(i-1, :), p(i, :));
    
        if(right_i == -2)
            cprintf(get_colour('final'), 'This is strange situation which should never occur... To be elaborated.');
            break;
        elseif (  (stat ~= -1) && ...
            ( (right_i == 0) || (norm(f_next - p(i-1, :)) < half_D_min) || ...
              (norm(f_next - p(i, :)) < half_D_min) ))
            % f_next is inside the current search segment
            % or it is outside, but very close to one of the edges
            % checking accuracy conditions
            
            % the angle between the vectors [p_{k-1}, p_k] and [p_k, p_next]
            ak = vector_angle(p(k, :) - p(k-1, :), p_next - p(k, :));
    
            if ((stat ~= 1) && (Dk*ak > Da_max))
                % the accuracy conditions are not met
                % note, that if angle range was enlarged, we do not check
                % this conditions
                if (Dk > D_min)
                    % decrease Dk
                    Dk = Dk/2;
                    if (Dk < half_D_min)
                        Dk = half_D_min;
                    end;
                    i = i_old;
                    dirctn = 0;
                    continue;
                else
                    % Dk is too small, accepting p_next as is
                    cprintf(get_colour('warn'), 'WARNING: the step size Dk = %.8f is too small, ', Dk);
                    cprintf(get_colour('warn'), 'accepting the point as is\n');
                    cprintf(get_colour('aux'), '(the angle ak = %.8f, the product Dk*ak = %.8f).\n\n', ak, Dk*ak);
                    break;
                end;
            else
                % the point is acceptable
                break;
            end % if ((ak > a_max) || (Dk*ak > Da_max))
        elseif ((stat ~= -1) && (right_i == 1))
            % f_next is mapped forward
            % searching segment forward
            if (dirctn == -1)
                cprintf(get_colour('warn'), ['WARNING: need to search forward, ' ...
                        'but already searched backward.\n']);
                if (Dk > D_min)
                    cprintf(get_colour('aux'), 'Decreasing step size...\n');
                    Dk = Dk/2;
                    if (Dk < half_D_min)
                        Dk = half_D_min;
                    end;
                    i = i_old;
                    dirctn = 0;
                    continue;
                else
                    cprintf(get_colour('warn'), 'WARNING: Step size is too small, aborting...\n');
                    cprintf(get_colour('final'), ['Manifold is computed upto %d point, ' ...
                            'arc length = %.8f\n'], k, arc_length);
                    return;
                end;
            end;
            dirctn = 1;
            i = i + 1;
            if (i > k)
                % no more segments to search forward
                cprintf(get_colour('warn'), 'WARNING: no more segments to search forward.\n');
                if (Dk > D_min)
                    cprintf(get_colour('aux'), 'Decreasing step size...\n');
                    Dk = Dk/2;
                    if (Dk < half_D_min)
                        Dk = half_D_min;
                    end;
                    i = i_old;
                    dirctn = 0;
                    continue;
                else
                    % step size is too small
                    
                    % check if the manifold ends at a periodic point
                    f_next1 = iterate(fhandle, f_next, iters, diverge, varargin{:});
                    if (norm(f_next1 - p_next) < half_D_min)
                        cprintf(get_colour('final'), '\n');
                        cprintf(get_colour('final'), ...
                                'The manifold seems to be of the finite arclength\n');
                        cprintf(get_colour('final'), ...
                                'The end point must be a periodic point of period %d: (%.8f, %.8f)\n', ...
                                iters, f_next1(1), f_next1(2));
                        cprintf(get_colour('final'), ...
                                'Manifold is computed upto %d point, arc length = %.8f.\n', ...
                                k, arc_length);
                        return;
                    end;
                    
                    % check if the manifold end at a periodic point of doubled period
                    f_next1 = iterate(fhandle, f_next1, iters, diverge, varargin{:});
                    if (norm(f_next1 - p_next) < half_D_min)
                        cprintf(get_colour('final'), '\n');
                        cprintf(get_colour('final'), ...
                                'The manifold seems to be of the finite arclength\n');
                        cprintf(get_colour('final'), ...
                                'The end point must be a periodic point of period %d: (%.8f, %.8f)\n', ...
                                2*iters, f_next1(1), f_next1(2));
                        cprintf(get_colour('final'), ...
                                'Manifold is computed upto %d point, arc length = %.8f.\n', ...
                                k, arc_length);
                        return;
                    end;

                    cprintf(get_colour('warn'), 'WARNING: Step size is too small, aborting...\n');
                    cprintf(get_colour('final'), ['Manifold is computed upto %d point, ' ...
                            'arc length = %.8f\n'], k, arc_length);
                    return;
                end;
            end;
            continue;
        else % (right_i == -1) || (stat == -1)
            % either f_next is mapped backward
            % or the solution was not found
            % searching segment backward
            if ((dirctn == 1) && (stat ~= -1))
                cprintf(get_colour('warn'), ['WARNING: need to search backward, ' ...
                        'but already searched forward.\n']);
                if (Dk > D_min)
                    cprintf(get_colour('aux'), 'Decreasing step size...\n');
                    Dk = Dk/2;
                    if (Dk < half_D_min)
                        Dk = half_D_min;
                    end;
                    i = i_old;
                    dirctn = 0;
                    continue;
                else
                    cprintf(get_colour('warn'), 'WARNING: Step size is too small, aborting...\n');
                    cprintf(get_colour('final'), ['Manifold is computed upto %d point, ' ...
                            'arc length = %.8f\n'], k, arc_length);
                    return;
                end;
            end;
            dirctn = -1;
            i = i - 1;
            if ((i < 2) || (stat == -1))
                % no more segments to search backward
                % or the candidate was not found
                cprintf(get_colour('warn'), 'WARNING: no more segments to search backward.\n');
                cprintf(get_colour('aux'), 'Current k = %d.\n', k);
                iters = mod(iters, max_iter) + 1;
                while (iters < min_iter)
                    iters = iters + 1;
                end;
                if (iters == iters_cur)
                    % we already tried this number of iterates
                    cprintf(get_colour('warn'), ...
                            'WARNING: We have already tried this number of iterates, aborting...\n');
                    cprintf(get_colour('final'), ['Manifold is computed upto %d point, ' ...
                            'arc length = %.8f\n'], k, arc_length);
                    return;
                else
                    cprintf(get_colour('warn'), ...
                            'Changing the number of iterates to %d.\n', iters);
                    i = i_old;
                    dirctn = 0;
                    continue;
                end;
            end;
            continue;
        end
        
    end % while (true)
    
    if(right_i == -2)
        break;
    end
    
    p(k+1, :) = p_next;
    arc_length = arc_length + Dk;
        
    % the step is too short: doubling
    if ((ak < a_min) && (Dk*ak < Da_min))
        Dk = 2*Dk;
    end;
    
    k = k + 1;

end % while (arc_length < arc_max), main section

cprintf(get_colour('final'), '\n');
cprintf(get_colour('final'), 'Computation finished.\n');
cprintf(get_colour('final'), 'Manifold is computed upto %d point, arc length = %.8f\n', k, arc_length);



